/*
********************************************************************************
sram23K256.h

Driver for the Microchip 23K256 32kbyte SRAM.

http://volker.top.geek.nz/arduino/

Copyright 2011 Volker Kuhlmann.
http://volker.top.geek.nz/contact.html

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

History:
 v1.0  07 Jan 2011  Created
 v1.1  08 Jan 2011  Finished, tested, ready for release.
********************************************************************************
*/

#ifndef SRAM23K256_H_
#define SRAM23K256_H_


extern "C" {
#include <inttypes.h>
}

// B... macros
#include <binary.h>


// Memory size in bytes.
#define SRAM23K256_MEMSIZE 32768

// Page size in bytes.
#define SRAM23K256_PAGESIZE 32

// Modus of operation.
#define SRAM23K256_READ   B00000011
#define SRAM23K256_WRITE  B00000010
#define SRAM23K256_RDSR   B00000101
#define SRAM23K256_WRSR   B00000001

// Status register bits.
#define SRAM23K256_SR_MODE      B11000000
#define SRAM23K256_SR_HOLD      B00000001

// Status register settings.
#define SRAM23K256_SR_BYTEMODE  B00000000
#define SRAM23K256_SR_PAGEMODE  B10000000
#define SRAM23K256_SR_SEQUMODE  B01000000
#define SRAM23K256_SR_HOLDDISABLE B00000001


class SRAM23K256
{
  public:
    void begin(uint8_t CSpin);
    void begin(void);
    void end(void);

    void enable(void);
    void disable(void);
    void reenable(void);

    uint8_t readstatus(void);
    void writestatus(uint8_t data);

    void sequential(void);
    void page(void);
    void samebyte(void);

    uint8_t read(uint16_t address);
    void write(uint16_t address, uint8_t data);
    void readstart(uint16_t address);
    void writestart(uint16_t address);
    
    void clear(void);

    void hold_enable(void);
    void hold_disable(void);

  private:
    uint8_t _cs;
    
    uint8_t _SPCR;
    uint8_t _SPSR;
};

#endif

/******************************************************************************/

